@@ -2,47 +2,46 @@ measurementInvarianceCat <- function(..., std.lv = FALSE, strict=FALSE, quiet=FA
                                    fit.measures = "default",
                                    method = "satorra.bentler.2001") {
 	List <- list(...)
-	if(!is.null(List$parameterization) && tolower(List$parameterization) != "theta") warnings("The parameterization is set to 'theta' by default.")
+	lavaancfa <- function(...) { lavaan::cfa(...) }
+	lavaanlavaan <- function(...) { lavaan::lavaan(...) }
+	if(!is.null(List$parameterization) && tolower(List$parameterization) != "theta") warning("The parameterization is set to 'theta' by default.")
 	List$parameterization <- "theta"
 
 	# Find the number of groups
-	ngroups <- 1
-	if(!is.null(List$group)) {
-		group <- List$group
-		ngroups <- length(setdiff(unique(List$data[,group]), NA))
-	} else {
-		stop("Please specify the group variable")
-	}
-	
+	if(is.null(List$group)) stop("Please specify the group variable")
+
 	# Get the lavaan parameter table
-	template <- do.call("cfa", c(List, do.fit=FALSE))
-    lavaanParTable <- lavaan::parTable(template)
-	
+	template <- do.call(lavaancfa, c(List, do.fit=FALSE))
+	lavaanParTable <- lavaan::parTable(template)
+
+	# Find the number of groups
+	ngroups <- max(lavaanParTable$group)
+
 	# Check whether all variables are categorical
 	sampstat <- inspect(template, "samp")[[1]]
 	meanname <- names(sampstat$mean)
 	thname <- names(sampstat$th)
 	if(any(is.na(charmatch(meanname, thname)))) stop("Some variables in your model are not identified as categorical.")
-	
+
 	varList <- lavaanParTable$rhs[lavaanParTable$op == "=~"]
 	facName <- lavaanParTable$lhs[(lavaanParTable$op == "=~") & (lavaanParTable$rhs %in% varList)]
 	if(length(unique(sapply(facName, function(x) length(x)))) > 1) stop("The numbers of variables in each element are not equal.")
 	varList <- unique(varList)
 	facName <- unique(facName)
-	
+
 	# Check whether the factor configuration is the same across gorups
 	groupParTable <- split(lavaanParTable, lavaanParTable$group)
 	group1pt <- groupParTable[[1]]
 	groupParTable <- lapply(groupParTable, "[", c("lhs", "op", "rhs"))
 	if(!multipleAllEqualList(lapply(groupParTable, function(x) sapply(x, "[", x$op == "=~")))) stop("Factor configuration is not the same across groups")
-	
+
 	# Extract the number of thresholds
 	numThreshold <- table(sapply(group1pt, "[", group1pt$op == "|")[,"lhs"])
-	
+
 	# Find the indicators of each factor
 	group1facload <- sapply(group1pt, "[", group1pt$op == "=~")
 	factorRep <- split(group1facload[,"rhs"], group1facload[,"lhs"])
-	
+
 	# Find marker variables
 	marker <- rep(NA, length(factorRep))
 	numThresholdMarker <- rep(NA, length(factorRep))
@@ -51,7 +50,7 @@ measurementInvarianceCat <- function(..., std.lv = FALSE, strict=FALSE, quiet=FA
 		marker[i] <- temp[!is.na(temp[,"ustart"]), "rhs"]
 		numThresholdMarker[i] <- numThreshold[marker[i]]
 	}
-	
+
 	numThresholdFactorRep <- lapply(factorRep, function(x) numThreshold[x])
 	constraintSecondThreshold <- unlist(lapply(numThresholdFactorRep, function(x) names(which(x > 1)[1])))
 	constraintSecondThreshold <- constraintSecondThreshold[!is.na(constraintSecondThreshold)]
@@ -60,13 +59,13 @@ measurementInvarianceCat <- function(..., std.lv = FALSE, strict=FALSE, quiet=FA
 	for(i in names(numThreshold)) {
 		lavaanParTable <- constrainParTable(lavaanParTable, i, "|", "t1", 1:ngroups)
 	}
-	
+
 	if(length(constraintSecondThreshold) > 0) {
 		for(i in constraintSecondThreshold) {
 			lavaanParTable <- constrainParTable(lavaanParTable, i, "|", "t2", 1:ngroups)
 		}
 	}
-	
+
 	# Group 1
 	for(i in facName) {
 		lavaanParTable <- fixParTable(lavaanParTable, i, "~1", "", 1, 0) # Fix factor means as 0
@@ -80,7 +79,7 @@ measurementInvarianceCat <- function(..., std.lv = FALSE, strict=FALSE, quiet=FA
 	for(i in varList) {
 		lavaanParTable <- fixParTable(lavaanParTable, i, "~~", i, 1, 1)
 	}
-	
+
 	# Other groups
 	for(k in 2:ngroups) {
 		for(i in facName) {
@@ -90,7 +89,7 @@ measurementInvarianceCat <- function(..., std.lv = FALSE, strict=FALSE, quiet=FA
 			} else {
 				lavaanParTable <- freeParTable(lavaanParTable, i, "~~", i, k, NA)
 			}
-		}	
+		}
 		for(i in varList) {
 			lavaanParTable <- freeParTable(lavaanParTable, i, "~~", i, k, NA)
 		}
@@ -99,7 +98,7 @@ measurementInvarianceCat <- function(..., std.lv = FALSE, strict=FALSE, quiet=FA
 			if(numThresholdMarker[i] == 1)  lavaanParTable <- fixParTable(lavaanParTable, marker[i], "~~", marker[i], k, 1)
 		}
 	}
-	
+
 	if(std.lv) {
 		for(i in seq_along(factorRep)) {
 			lavaanParTable <- freeParTable(lavaanParTable, names(factorRep)[i], "=~", marker[i], 1:ngroups, NA)
@@ -108,11 +107,11 @@ measurementInvarianceCat <- function(..., std.lv = FALSE, strict=FALSE, quiet=FA
 	# Fit configural invariance
 	ListConfigural <- List
 	ListConfigural$model <- lavaanParTable
-	fitConfigural <- do.call("lavaan", ListConfigural)
-	
+	fitConfigural <- do.call(lavaanlavaan, ListConfigural)
+
 	# Create the parameter table for metric invariance
 	ptMetric <- lavaanParTable
-	
+
 	for(i in seq_along(factorRep)) {
 		varwithin <- factorRep[[i]]
 		if(!std.lv) {
@@ -129,13 +128,13 @@ measurementInvarianceCat <- function(..., std.lv = FALSE, strict=FALSE, quiet=FA
 			}
 		}
 	}
-	
+
 	ListMetric <- List
 	ListMetric$model <- ptMetric
-	fitMetric <- do.call("lavaan", ListMetric)
+	fitMetric <- do.call(lavaanlavaan, ListMetric)
 
 	ptMeans <- ptStrict <- ptMetric
-	
+
 	nonMarker <- setdiff(names(numThreshold), marker)
 	nonDichoMarker <- numThreshold[which(numThreshold[nonMarker] > 1)]
 	scalar <- length(nonDichoMarker) > 0
@@ -149,10 +148,10 @@ measurementInvarianceCat <- function(..., std.lv = FALSE, strict=FALSE, quiet=FA
 		}
 		ListScalar <- List
 		ListScalar$model <- ptScalar
-		fitScalar <- do.call("lavaan", ListScalar)		
+		fitScalar <- do.call(lavaanlavaan, ListScalar)
 		ptMeans <- ptStrict <- ptScalar
 	}
-	
+
 
 	fitStrict <- NULL
 	# Create the parameter table for strict invariance if specified
@@ -166,19 +165,19 @@ measurementInvarianceCat <- function(..., std.lv = FALSE, strict=FALSE, quiet=FA
 		}
 		ListStrict <- List
 		ListStrict$model <- ptStrict
-		fitStrict <- do.call("lavaan", ListStrict)		
+		fitStrict <- do.call(lavaanlavaan, ListStrict)
 		ptMeans <- ptStrict
-	} 
-	
+	}
+
 	# Create the parameter table for mean equality
-	
+
 	# Constrain factor means to be equal
 	for(k in 2:ngroups) {
 		ptMeans <- fixParTable(ptMeans, facName, "~1", "", k, ustart = 0)
 	}
 	ListMeans <- List
 	ListMeans$model <- ptMeans
-	fitMeans <- do.call("lavaan", ListMeans)		
+	fitMeans <- do.call(lavaanlavaan, ListMeans)
 
 	FIT <- invisible(list(fit.configural = fitConfigural, fit.loadings = fitMetric, fit.thresholds = fitScalar, fit.residuals = fitStrict, fit.means = fitMeans))
 	FIT <- FIT[!sapply(FIT, is.null)]
@@ -199,12 +198,12 @@ multipleAllEqualList <- function(obj) {
     for (i in 2:length(obj)) {
         for (j in 1:(i - 1)) {
             temp <- isTRUE(all.equal(obj[[i]], obj[[j]]))
-            if (!temp) 
+            if (!temp)
                 return(FALSE)
         }
     }
     return(TRUE)
-} 
+}
 
 multipleAnyEqual <- function(...) {
     obj <- list(...)
@@ -215,11 +214,11 @@ multipleAnyEqualList <- function(obj) {
     for (i in 2:length(obj)) {
         for (j in 1:(i - 1)) {
             temp <- isTRUE(all.equal(obj[[i]], obj[[j]]))
-            if (temp) 
+            if (temp)
                 return(TRUE)
         }
     }
     return(FALSE)
-} 
+}
 
 
