@@ -6,24 +6,24 @@
 
 ##Currently outputs a list of parameter estimates, standard errors, fit indices and fraction missing information
 
-cfa.mi <- function(model, data, m, miArgs=list(), miPackage="Amelia", chi="all", seed=12345, nullModel = NULL, ...) {
-	runMI(model=model, data=data, m=m, miArgs=miArgs, chi=chi, miPackage=miPackage, seed=seed, fun="cfa", nullModel = nullModel, ...)
+cfa.mi <- function(model, data, m, miArgs=list(), miPackage="Amelia", chi="all", seed=12345, nullModel = NULL, includeImproper = FALSE, ...) {
+	runMI(model=model, data=data, m=m, miArgs=miArgs, chi=chi, miPackage=miPackage, seed=seed, fun="cfa", nullModel = nullModel, includeImproper = includeImproper, ...)
 }
 
-sem.mi <- function(model, data, m, miArgs=list(), miPackage="Amelia", chi="all", seed=12345, nullModel = NULL, ...) {
-	runMI(model=model, data=data, m=m, miArgs=miArgs, chi=chi, miPackage=miPackage, seed=seed, fun="sem", nullModel = nullModel, ...)
+sem.mi <- function(model, data, m, miArgs=list(), miPackage="Amelia", chi="all", seed=12345, nullModel = NULL, includeImproper = FALSE, ...) {
+	runMI(model=model, data=data, m=m, miArgs=miArgs, chi=chi, miPackage=miPackage, seed=seed, fun="sem", nullModel = nullModel, includeImproper = includeImproper, ...)
 }
 
-growth.mi <- function(model, data, m, miArgs=list(), miPackage="Amelia", chi="all", seed=12345, nullModel = NULL, ...) {
-	runMI(model=model, data=data, m=m, miArgs=miArgs, chi=chi, miPackage=miPackage, seed=seed, fun="growth", nullModel = nullModel, ...)
+growth.mi <- function(model, data, m, miArgs=list(), miPackage="Amelia", chi="all", seed=12345, nullModel = NULL, includeImproper = FALSE, ...) {
+	runMI(model=model, data=data, m=m, miArgs=miArgs, chi=chi, miPackage=miPackage, seed=seed, fun="growth", nullModel = nullModel, includeImproper = includeImproper, ...)
 }
 
-lavaan.mi <- function(model, data, m, miArgs=list(), miPackage="Amelia", chi="all", seed=12345, nullModel = NULL, ...) {
-	runMI(model=model, data=data, m=m, miArgs=miArgs, chi=chi, miPackage=miPackage, seed=seed, fun="lavaan", nullModel = nullModel, ...)
+lavaan.mi <- function(model, data, m, miArgs=list(), miPackage="Amelia", chi="all", seed=12345, nullModel = NULL, includeImproper = FALSE, ...) {
+	runMI(model=model, data=data, m=m, miArgs=miArgs, chi=chi, miPackage=miPackage, seed=seed, fun="lavaan", nullModel = nullModel, includeImproper = includeImproper, ...)
 }
 
 
-runMI <- function(model, data, m, miArgs=list(), chi="all", miPackage="Amelia", seed=12345, fun, nullModel = NULL, ...) 
+runMI <- function(model, data, m, miArgs=list(), chi="all", miPackage="Amelia", seed=12345, fun, nullModel = NULL, includeImproper = FALSE, ...) 
 {
 	set.seed(seed)
 	chi <- tolower(chi)
@@ -50,7 +50,6 @@ runMI <- function(model, data, m, miArgs=list(), chi="all", miPackage="Amelia",
 	out <- list(model=model, data=imputed.l[[1]], se="none", do.fit=FALSE)
 	out <- c(out, list(...))
 	template <- do.call(fun, out)
-
     imputed.results.l <- suppressWarnings(lapply(imputed.l, runlavaanMI, syntax=model, fun=fun, ...))
 	converged.l <- sapply(imputed.results.l, function(x) x@Fit@converged)
 	
@@ -62,7 +61,9 @@ runMI <- function(model, data, m, miArgs=list(), chi="all", miPackage="Amelia",
 	properSE <- apply(seAll, 2, function(x) all(!is.na(x)) & all(x >= 0) & !(all(x == 0)))
 	
 	properVariance <- apply(coefAll[posVar, ,drop=FALSE], 2, function(x) all(x >= 0))
-	converged.l <- converged.l & properSE & properVariance
+	if(!includeImproper) {
+		converged.l <- converged.l & properSE & properVariance
+	}
 	if(sum(converged.l) < 2) {
 		tab <- cbind(convergedtemp, properSE, properVariance, converged.l)
 		colnames(tab) <- c("1. Convergence", "2. Proper SE", "3. Proper Variance Estimate", "Used for pooling")
@@ -90,16 +91,24 @@ runMI <- function(model, data, m, miArgs=list(), chi="all", miPackage="Amelia",
 	template@Fit@Sigma.hat <- meanSigmaHat
 	template@Fit@Mu.hat <- meanMuHat
 	comb.results <- miPoolVector(t(coefs),t(se), m)
+	
+	
 	template@Fit@est <- template@ParTable$est <- comb.results$coef
 	template@Fit@se <- template@ParTable$se <- comb.results$se
 	template@Fit@x <- comb.results$coef[comb.results$se != 0]
 	template@Model <- imposeGLIST(template@Model, comb.results$coef, template@ParTable)
+
+	selectVCOV <- lavaan::partable(imputed.results.l[[1]])$free != 0
+	# VCOV
+	VCOVs <- sapply(imputed.results.l, function(x) vecsmat(vcov(x)))
+	
+	template@vcov$vcov <- vcovPool(t(coefs[selectVCOV, ]),t(VCOVs), m)
 	
 	fmi.results <- cbind(lavaan::parameterEstimates(template, remove.system.eq = FALSE, remove.eq = FALSE, remove.ineq = FALSE)[,1:3], group=template@ParTable$group, fmi1 = comb.results[[3]], fmi2 = comb.results[[4]])
 
 	fit <- imputed.results.l[[1]]@test
 	df <- fit[[1]]$df
-  if (df == 0) chi <- "none" # for saturated models, no model fit available
+  #if (df == 0) chi <- "none" # for saturated models, no model fit available
 	chi1 <- sapply(imputed.results.l, function(x) x@test[[1]]$stat)
 
 	if(any(template@Data@ov$type == "ordered") | (length(fit) > 1)) {
@@ -122,20 +131,24 @@ runMI <- function(model, data, m, miArgs=list(), chi="all", miPackage="Amelia",
 		fit[[1]]$stat.group <- rowMeans(sapply(imputed.results.l, function(x) x@test[[1]]$stat.group))
 	}
 	if(is.null(nullModel)) nullModel <- lavaan::lav_partable_independence(template)
-	
+	if(is.list(nullModel)) nullModel$ustart[nullModel$exo == 1] <- NA
 	
     null.results <- suppressWarnings(lapply(imputed.l, runlavaanMI, syntax=nullModel, fun=fun, ...))
 
 	convergedNull.l <- sapply(null.results, function(x) x@Fit@converged)
 	seNullAll <- sapply(null.results, function(x) x@Fit@se)
-	convergedNull.l <- convergedNull.l & apply(seNullAll, 2, function(x) all(!is.na(x) & (x >= 0)))
+	if(!includeImproper) {
+		convergedNull.l <- convergedNull.l & apply(seNullAll, 2, function(x) all(!is.na(x) & (x >= 0)))
+	}
+	
+	dfNull <- null.results[[1]]@test[[1]]$df
+	if(dfNull == 0) convergedNull.l <- rep(TRUE, m)
 	if(!any(convergedNull.l)) stop("No null model is converged")
 	
 	mNull <- sum(convergedNull.l)
 	convergenceNullRate <- mNull/mOriginal
 	null.results <- null.results[convergedNull.l]
 	chiNull <- sapply(null.results, function(x) x@test[[1]]$stat)
-	dfNull <- null.results[[1]]@test[[1]]$df
 	
 	chiNullScaled1 <- NULL
 	dfNullScaled <- NULL
@@ -153,7 +166,6 @@ runMI <- function(model, data, m, miArgs=list(), chi="all", miPackage="Amelia",
 	comb.results.null <- miPoolVector(t(coefsNull),t(seNull), mNull)
 	fitNull <- null.results[[1]]@test
 
-	
 	lmrr <- NULL
 	lmrrNull <- NULL
 	mr <- NULL
@@ -162,6 +174,11 @@ runMI <- function(model, data, m, miArgs=list(), chi="all", miPackage="Amelia",
 	mplusNull <- NULL
 	lmrrScaled <- NULL
 	lmrrScaledNull <- NULL
+	logsat <- NA
+	logalt <- NA
+	loglnull <- NULL
+	loglsat <- NULL
+	loglmod <- NULL
 	
 	if(chi %in% c("lmrr", "all")){
 		lmrr <- lmrrPooledChi(chi1, df)
@@ -188,8 +205,17 @@ runMI <- function(model, data, m, miArgs=list(), chi="all", miPackage="Amelia",
 	}
 	
 	if(chi %in% c("mplus", "mr", "all")){
-		mrplus <- mrplusPooledChi(template, imputed.l[converged.l], chi1, df, coef=comb.results$coef, m=m, fun=fun, ...)
-		mrplusNull <- mrplusPooledChi(templateNull, imputed.l[convergedNull.l], chiNull, dfNull, coef=comb.results.null$coef, m=mNull, fun=fun, par.sat=satPartable(template), ...)
+		mrplusOut <- mrplusPooledChi(template, imputed.l[converged.l], chi1, df, coef=comb.results$coef, coefs = coefs, m=m, fun=fun, ...)
+		mrplus <- mrplusOut[[1]]
+		mrplusChi <- mrplusOut[[2]]
+		mrplusNullOut <- mrplusPooledChi(templateNull, imputed.l[convergedNull.l], chiNull, dfNull, coef=comb.results.null$coef, coefs = coefsNull, m=mNull, fun=fun, par.sat=lavaan::lav_partable_unrestricted(template), ...)
+		mrplusNull <- mrplusNullOut[[1]]
+		mrplusNullChi <- mrplusNullOut[[2]]
+		logsat <- mrplus[5] / (1 + mrplus[4])
+		logalt <- mrplus[6] / (1 + mrplus[4])
+		loglnull <- mrplusNullChi[,2]
+		loglsat <- mrplusChi[,1]
+		loglmod <- mrplusChi[,2]
 		
 		if(chi %in% c("mr", "all")){
 			mr <- mrPooledChi(mrplus[1], mrplus[2], mrplus[3], mrplus[4])
@@ -231,14 +257,18 @@ runMI <- function(model, data, m, miArgs=list(), chi="all", miPackage="Amelia",
 	summaryImputed[[2]] <- fmi.results
 	summaryImputed[[3]] <- list(lmrr = lmrr, mr = mr, mplus = mplus)
 	summaryImputed[[4]] <- list(lmrr = lmrrNull, mr = mrNull, mplus = mplusNull)
+	summaryImputed[[5]] <- list(unrestricted.logl = logsat, logl = logalt)
+	summaryImputed[[6]] <- list(chiorig = chi1, loglmod = loglmod, loglnull = loglnull, loglsat = loglsat)
+	nameImputed <- c("convergenceRate", "fractionMissing", "targetFit", "nullFit", "logl", "indivlogl")
 	if(!is.null(lmrrScaled)) {
-		summaryImputed[[5]] <- list(lmrr = lmrrScaled)
-		summaryImputed[[6]] <- list(lmrr = lmrrScaledNull)
-		names(summaryImputed) <- c("convergenceRate", "fractionMissing", "targetFit", "nullFit", "targetFit.scaled", "nullFit.scaled")
+		summaryImputed[[7]] <- list(lmrr = lmrrScaled)
+		summaryImputed[[8]] <- list(lmrr = lmrrScaledNull)
+		names(summaryImputed) <- c(nameImputed, "targetFit.scaled", "nullFit.scaled")
 	} else {
-		names(summaryImputed) <- c("convergenceRate", "fractionMissing", "targetFit", "nullFit")
+		names(summaryImputed) <- nameImputed
 	}
 	result@imputed <- summaryImputed
+	result@imputedResults <- imputed.results.l
 
 	return(result)
 }
@@ -356,6 +386,37 @@ names(MI.res)<-c('coef','se','FMI.1','FMI.2')
 #			Sunthud Pornprasertmanit (University of Kansas; psunthud@ku.edu)
 # Date Modified: March 31, 2012
 
+vecsmat <- function(X) X[lower.tri(X, diag = TRUE)]
+
+invvecsmat <- function(x) {
+	p <- (sqrt(1 + 8 * length(x)) - 1) /2
+	X <- matrix(0, p, p)
+	X[lower.tri(X, diag = TRUE)] <- x
+	vars <- diag(X)
+	X <- X + t(X)
+	diag(X) <- vars
+	X
+}
+
+vcovPool <- function(MI.param, MI.cov, imps) {
+   #compute parameter estimates
+  Estimates <- colMeans(MI.param)
+
+#compute between-imputation variance: variance of parameter estimates
+  Bm <- vecsmat(cov(MI.param))
+
+#compute within-imputation variance: average of squared estimated SEs 
+#Um <- colSums(MI.se^2/m)
+  Um <- apply(MI.cov,2,mean)
+
+#Total variance
+#Tm <- Um + (Bm)*((1+m)/m+1)
+#compute total variance: sum of between- and within- variance with correction
+  TV <- Um + ((imps+1)/imps)*Bm
+
+  return(invvecsmat(TV))
+}
+
 lmrrPooledChi <- function(chis, df) {
 	# From Li, Meng, Raghunathan, & Rubin (1991)
 	if(is.matrix(chis)) {
@@ -380,48 +441,10 @@ lmrrPooledChi <- function(chis, df) {
 #Examples:
 #lmrrPooledChi(c(89.864, 81.116,71.500,49.022,61.986,64.422,55.256,57.890,79.416,63.944), 2)
 
-
-## function that builds a lavaan parameter table of the saturate model
-## using the information in a lavaan object from the cfa function
-satPartable <- function(fit.alt){
-  
-	par.alt <- lavaan::partable(fit.alt) #get the parameter table form the original model
-	ngroups <- fit.alt@Data@ngroups # get the number of groups 
-	# gets the parameter table from the null model
-	
-	par.null <- lavaan::lav_partable_independence(fit.alt)
-	lhs.diag <- par.null$lhs
-	op.diag <- par.null$op
-	rhs.diag <- par.null$rhs
-	gnull <- par.null$group
-	#combine the variable names to set al the covariances
-	pairs <- t(combn(lavaan::lavaanNames(fit.alt, type="ov"), 2))
-	lhs.up <- rep(pairs[, 1],times=ngroups)
-	op.up <- rep("~~", length(lhs.up))
-	rhs.up <- rep(pairs[, 2],times=ngroups)
-	galt <- sort(rep(1:ngroups,times=length(lhs.up)/ngroups))
-	#put together the null table and the covariances
-	lhs.all <- c(lhs.up, lhs.diag)
-	id <- seq(1:length(lhs.all))
-	op.all <- c(op.up, op.diag)
-	rhs.all <- c(rhs.up, rhs.diag)
-	user <- rep(1,length(lhs.all))
-	group <- as.integer(c(galt,gnull))
-	free <- as.integer(id)
-	ustart <- rep(NA, length(lhs.all))
-	exo <- rep(0, length(lhs.all))
-	label <- rep("", length(lhs.all))
-	plabel <- paste0(".p", id, ".")
-	par.sat <- list(id, lhs.all, op.all, rhs.all, user, group,
-				  free, ustart, exo, label, plabel)
-	names(par.sat) <- c("id", "lhs", "op", "rhs", "user", "group", "free", "ustart", "exo", "label", "plabel")
-	return(par.sat)
-}
-
 ##### function that does the part of the MR and Mplus combination methods are equal 
-mrplusPooledChi <- function(template, imputed.l, chi1, df, coef, m, fun, par.sat=NULL, ...) {
+mrplusPooledChi <- function(template, imputed.l, chi1, df, coef, coefs, m, fun, par.sat=NULL, ...) {
 	
-	if(is.null(par.sat)) par.sat <- satPartable(template)
+	if(is.null(par.sat)) par.sat <- lavaan::lav_partable_unrestricted(template) 
 	comb.sat <- suppressWarnings(lapply(imputed.l, runlavaanMI, syntax=par.sat, fun=fun, ...))
 	converged.sat1 <- sapply(comb.sat, function(x) x@Fit@converged)
 	
@@ -430,25 +453,41 @@ mrplusPooledChi <- function(template, imputed.l, chi1, df, coef, m, fun, par.sat
 	par.sat2 <- par.sat
 	par.sat2$free <- as.integer(rep(0, length(par.sat2$free)))
 	par.sat2$ustart <- est.sat1
+	par.sat2$start <- est.sat1
+	par.sat2$est <- est.sat1
 	comb.sat2 <- suppressWarnings(lapply(imputed.l, runlavaanMI, syntax=par.sat2, fun=fun, ...))
-	
     comb.sat2 <- lapply(comb.sat2, forceTest)
 	fit.sat2 <- sapply(comb.sat2, function(x) inspect(x, "fit")["logl"])
-	
+
 	par.alt2 <- lavaan::partable(template)
 	par.alt2$free <- as.integer(rep(0, length(par.alt2$free)))
 	par.alt2$ustart <- coef
-	comb.alt2 <- suppressWarnings(lapply(imputed.l, runlavaanMI, syntax=par.alt2, fun=fun, ...))
+	par.alt2$start <- coef
+	par.alt2$est <- coef
+	par.alt2.l <- rep(list(par.alt2), m)
+	TEMPFUN <- function(ptable, origcoef) {
+		exo <- ptable$exo == 1
+		ptable$ustart[exo] <- origcoef[exo]
+		ptable$start[exo] <- origcoef[exo]
+		ptable$est[exo] <- origcoef[exo]
+		ptable
+	}
+	par.alt2.l <- mapply(TEMPFUN, par.alt2.l, data.frame(coefs), SIMPLIFY = FALSE)
+	comb.alt2 <- suppressWarnings(mapply(runlavaanMI, MIdata = imputed.l, syntax = par.alt2.l, SIMPLIFY = FALSE, MoreArgs = list(fun = fun, ...)))
+	#comb.alt2 <- suppressWarnings(lapply(imputed.l, runlavaanMI, syntax=par.alt2, fun=fun, ...))
     comb.alt2 <- lapply(comb.alt2, forceTest)
 	fit.alt2 <- sapply(comb.alt2, function(x) inspect(x, "fit")["logl"])
-  	
 	chinew <- cbind(fit.sat2, fit.alt2, (fit.sat2-fit.alt2)*2)
+	
+	
 	chimean <- mean(chinew[,3])
+	logsat <- mean(chinew[,1])
+	logalt <- mean(chinew[,2])
 	
 	fit.altcc <- mean(chi1)
 	ariv <- ((m+1)/((m-1)*df))*(fit.altcc-chimean)
-	resmrCHI <- c(chimean, m, df, ariv)
-	return(resmrCHI)
+	resmrCHI <- c(chimean, m, df, ariv, logsat, logalt)
+	return(list(resmrCHI, chinew))
 }
 
 ##### function that does the calculations for the Mplus chi combination
